@@ -21,13 +21,15 @@
   (interactive)
   (let ((opoint (point))
 	;; In Emacs 21, during a minibuffer read the minibuffer
-	;; contains propt as buffer text and that text is read
-	;; only.  So we can no longer assume that (point-min) is
-	;; where the user-entered text starts and we must avoid
-	;; modifying that prompt text.  Calling
-	;; previous-property-change is a kludge but it does the
-	;; job.
-	(point-min (previous-property-change (point) nil (point-min)))
+	;; contains the prompt as buffer text and that text is
+	;; read only.  So we can no longer assume that (point-min)
+	;; is where the user-entered text starts and we must avoid
+	;; modifying that prompt text.  The value we want instead
+	;; of (point-min) is (minibuffer-prompt-end).
+	(point-min (if (fboundp 'minibuffer-prompt-end)
+		       (minibuffer-prompt-end)
+		     (point-min)))
+	(case-fold-search completion-ignore-case)
 	trimmed-c-list c-list beg end diff word word-prefix-regexp completion)
     ;; find the beginning and end of the word we're trying to complete
     (if (or (eobp) (memq (following-char) '(?\t ?\n ?\ )))
@@ -86,9 +88,11 @@
       (setq diff (- (length completion) (length word)))
       (cond
        ;; We have some completion chars.  Insert them.
-       ((> diff 0)
+       ((or (> diff 0)
+	    (and (zerop diff) (not (string-equal completion word))))
 	(goto-char end)
-	(insert (substring completion (- diff)))
+	(delete-char (- (length word)))
+	(insert completion)
 	(if (and vm-completion-auto-space
 		 (null (cdr trimmed-c-list)))
 	    (insert " ")))
